@@ -5,9 +5,15 @@
 #include <random>
 
 // [[Rcpp::export]]
-SEXP GWAS(Eigen::MatrixXf Y, Eigen::MatrixXf GEN, Eigen::MatrixXf M,
-          int NumPCs = 3, int maxit = 500, float logtol=-8,
-          int cores = 1, bool verb = true){
+SEXP GWAS(Eigen::MatrixXf Y, 
+          Eigen::MatrixXf GEN,
+          Eigen::MatrixXf M,
+          int NumPCs = 3, 
+          int maxit = 500, 
+          float logtol=-8,
+          int cores = 1, 
+          bool verb = false)
+  {
   
   // 
   // Null model - Multivariate SNP-BLUP
@@ -76,7 +82,7 @@ SEXP GWAS(Eigen::MatrixXf Y, Eigen::MatrixXf GEN, Eigen::MatrixXf M,
   Eigen::MatrixXf LHS(k,k);
   Eigen::VectorXf RHS(k);
   Eigen::MatrixXf b = Eigen::MatrixXf::Zero(p,k);
-  Eigen::VectorXf b0(k), b1(k);
+  Eigen::VectorXf b0(k), b1(k), e2(n0);
   Eigen::MatrixXf e(n0,k); e = y*1.0;
   // Bending
   float deflate = 1.0, deflateMax = 0.9;
@@ -249,13 +255,19 @@ SEXP GWAS(Eigen::MatrixXf Y, Eigen::MatrixXf GEN, Eigen::MatrixXf M,
       // Compute coefficient
       ZPZ = ZM.transpose() * PZM; 
       ZPZ.diagonal().array() += 0.1; // Added for algorithmic stability
-      ZPy = ZM.transpose() * e.col(trait);
+      
+      //ZPy = ZM.transpose() * e.col(trait);
+      // Add back the marker effect from polygenic term 11/05/2025
+      e2 = (e.col(trait).array() + (GEN.col(J)*beta(J,trait)).array())*Z.col(trait).array();
+      ZPy = ZM.transpose() * e2;
+      
       gamma = ZPZ.bdcSvd(Eigen::ComputeThinU|Eigen::ComputeThinV).solve(ZPy);
       Gamma.row(J) = gamma.array();
       
       // Variance components and heritability
       yZg = (y.col(trait).array()-(ZM*gamma).array())* Z.col(trait).array();
       PyZg = e.col(trait).array()-(PZM*gamma).array();
+      
       VarE = (yZg.transpose()*PyZg);
       VarE = VarE*dff;
       tmp = (gamma.transpose() * ZPZ * gamma).trace();
@@ -298,160 +310,3 @@ SEXP GWAS(Eigen::MatrixXf Y, Eigen::MatrixXf GEN, Eigen::MatrixXf M,
   return OutputList;
   
 }
-
-
-// [[Rcpp::export]]
-SEXP MLM(Eigen::MatrixXf Y, Eigen::MatrixXf X, Eigen::MatrixXf Z,
-         int maxit = 500, float logtol = -8, int cores = 1){
-  
-  // Basic info
-  float df0 = 1.1; 
-  if(cores!=1) Eigen::setNbThreads(cores);
-  int k = Y.cols(), n0 = Y.rows(), f = X.cols(), p = Z.cols();
-  
-  // Incidence matrix W
-  Eigen::MatrixXf W(n0,k);
-  for(int i=0; i<n0; i++){
-    for(int j=0; j<k; j++){
-      if(std::isnan(Y(i,j))){
-        W(i,j) = 0.0;
-        Y(i,j) = 0.0;
-      }else{ W(i,j) = 1.0;}}}
-  Eigen::VectorXf n = W.colwise().sum();
-  Eigen::VectorXf iN = (n.array()-f).inverse();
-  
-  // Compute SY, SZ
-  Eigen::MatrixXf y(n0,k), WX(n0,f), MU(f,k), iXX(f,f);
-  for(int i=0; i<k; i++){
-    for(int j=0; j<f; j++){WX.col(j)=X.col(j).array()*W.col(i).array();}
-    iXX = (WX.transpose()*WX).inverse();
-    MU.col(i) = iXX * WX.transpose()*Y.col(i);
-    y.col(i) = (Y.col(i)-WX*MU.col(i) ).array()*W.col(i).array(); }
-  iXX = (X.transpose()*X).inverse();
-  for(int j=0; j<p; j++){ Z.col(j) = (Z.col(j) - X*(iXX*X.transpose()*Z.col(j))).array(); }
-  
-  // Sum of squares of Z
-  Eigen::MatrixXf ZZ(p,k); 
-  for(int i=0; i<p; i++){ ZZ.row(i) = Z.col(i).array().square().matrix().transpose() * W;}
-  Eigen::VectorXf TrZSZ = ZZ.colwise().sum().array();
-  
-  // Initialize coefficient matrices
-  Eigen::MatrixXf LHS(k,k);
-  Eigen::VectorXf RHS(k);
-  Eigen::MatrixXf b = Eigen::MatrixXf::Zero(p,k);
-  Eigen::VectorXf b0(k), b1(k);
-  Eigen::MatrixXf e(n0,k); e = y*1.0;
-  
-  // Variances
-  Eigen::VectorXf vy = y.colwise().squaredNorm(); vy=vy.array()*iN.array();
-  Eigen::VectorXf ve = vy * 0.5;
-  Eigen::VectorXf iVe = ve.array().inverse();
-  Eigen::MatrixXf vb(k,k), TildeHat(k,k);
-  vb = (ve.array()/ (TrZSZ.array()*iN.array())  ).matrix().asDiagonal();
-  Eigen::MatrixXf iG = vb.inverse();
-  Eigen::VectorXf h2 = 1 - ve.array()/vy.array();
-  Eigen::MatrixXf tilde = Z.transpose() * y;
-  
-  // Bending
-  float deflate = 1.0, deflateMax = 0.9;
-  Eigen::MatrixXf A = vb*1.0;
-  Eigen::SelfAdjointEigenSolver<Eigen::MatrixXf> EVDofA(A);
-  float MinDVb, inflate = 0.0;
-  
-  // Prior for stability
-  Eigen::MatrixXf Sb = vb*df0;
-  Eigen::VectorXf Se = ve*df0;
-  Eigen::VectorXf iNp = (n.array()+df0-f).inverse();
-  
-  // RGS
-  std::vector<int> RGSvec(p);
-  for(int j=0; j<p; j++){RGSvec[j]=j;}
-  std::random_device rd;
-  std::mt19937 g(rd());
-  int J;
-  
-  // Convergence control
-  Eigen::MatrixXf beta0(p,k);
-  float cnv = 10.0;
-  int numit = 0;
-  
-  // Loop
-  while(numit<maxit){
-    
-    // Store coefficients pre-iteration
-    beta0 = b*1.0;
-    
-    // Randomized Gauss-Seidel loop
-    std::shuffle(RGSvec.begin(), RGSvec.end(), g);
-    for(int j=0; j<p; j++){
-      J = RGSvec[j];
-      
-      // Update coefficient
-      b0 = b.row(J)*1.0;
-      LHS = iG;  LHS.diagonal() += (ZZ.row(J).transpose().array() * iVe.array()).matrix();
-      RHS = (Z.col(J).transpose()*e).array() + ZZ.row(J).array()*b0.transpose().array();
-      RHS = RHS.array() *iVe.array();
-      b1 = LHS.llt().solve(RHS);
-      b.row(J) = b1;
-      
-      // Update residuals
-      e = (e-(Z.col(J)*(b1-b0).transpose()).cwiseProduct(W)).matrix();}
-    
-    // Residual variance
-    ve = (e.cwiseProduct(y)).colwise().sum();
-    //ve = ve.array() * iN.array();
-    ve = (ve.array()+Se.array()) * iNp.array();
-    iVe = ve.array().inverse();
-    
-    // Genetic variance
-    TildeHat = b.transpose()*tilde;
-    for(int i=0; i<k; i++){for(int j=0; j<k; j++){
-      if(i==j){
-        vb(i,i) = (TildeHat(i,i)+Sb(i,i))/(TrZSZ(i)+df0);            }else{
-          vb(i,j) = (TildeHat(i,j)+TildeHat(j,i))/(TrZSZ(i)+TrZSZ(j)); }}}
-    
-    // Bending
-    A = vb*1.0;
-    for(int i=0; i<k; i++){ for(int j=0; j<k; j++){if(i!=j){A(i,j) *= deflate;}}}
-    if(A.llt().info()==Eigen::NumericalIssue && deflate>deflateMax){
-      deflate -= 0.005; Rcpp::Rcout <<  deflate;
-      for(int i=0; i<k; i++){for(int j=0; j<k; j++){if(i!=j){A(i,j) = vb(i,j)*deflate;}}}}
-    EVDofA.compute(A);
-    MinDVb = EVDofA.eigenvalues().minCoeff();
-    if( MinDVb < 0.0 ){ 
-      Rcpp::Rcout << ".";
-      inflate = inflate - MinDVb*1.001;
-      A.diagonal().array() += inflate;}
-    iG = A.inverse();
-    
-    // Print status
-    ++numit;
-    cnv = log10((beta0.array()-b.array()).square().sum());
-    if( std::isnan(cnv) ){Rcpp::Rcout << "Numerical issue! Job aborted (it=" << numit << ")\n"; break;}
-    if( numit % 100 == 0 ){ Rcpp::Rcout << "Iter: "<< numit << " || log10 Conv: "<< cnv << "\n"; } 
-    if(  cnv<logtol ){ Rcpp::Rcout << "Model coverged in "<< numit << " iterations\n"; break; }
-    if( numit == maxit ){ Rcpp::Rcout << "Model did not converge\n"; }
-    
-  }
-  
-  // Fitting the model
-  h2 = 1 - ve.array()/vy.array();
-  Eigen::MatrixXf hat = Z*b;
-  for(int i=0; i<k; i++){ hat.col(i) = ( X * MU.col(i) + hat.col(i)).array(); }
-  
-  // Genetic correlations
-  Eigen::MatrixXf GC(k,k);
-  for(int i=0; i<k; i++){for(int j=0; j<k; j++){GC(i,j)=vb(i,j)/(sqrt(vb(i,i)*vb(j,j)));}}
-  
-  // Name and create outputs
-  Rcpp::List OutputList = Rcpp::List::create(Rcpp::Named("b")=MU,Rcpp::Named("u")=b,
-                                             Rcpp::Named("hat")=hat,Rcpp::Named("h2")=h2,
-                                             Rcpp::Named("GC")=GC,Rcpp::Named("bend")=deflate,
-                                             Rcpp::Named("vb")=vb,Rcpp::Named("ve")=ve,
-                                             Rcpp::Named("cnv")=cnv,Rcpp::Named("its")=numit);
-  
-  // Output
-  OutputList.attr("class") = "PEGSmodel";
-  return OutputList;
-  
-}
